using System.Collections.Generic;
using System.Collections.Immutable;
using Upsilon.BaseTypes;
using Upsilon.BaseTypes.ScriptFunction;
using Upsilon.Evaluator;
using Upsilon.Exceptions;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public class BoundFunctionCallExpression : BoundExpression
    {
        public BoundExpression Identifier { get; }
        public ImmutableArray<BoundExpression> Parameters { get; }

        public BoundFunctionCallExpression(BoundExpression identifier, ImmutableArray<BoundExpression> parameters,
            TextSpan span, TypeContainer type) : base(span)
        {
            Identifier = identifier;
            Parameters = parameters;
            ValueType = type;
        }

        public override BoundKind Kind => BoundKind.BoundFunctionCallExpression;

        public override IEnumerable<BoundNode> GetChildren()
        {
            yield return Identifier;
            foreach (var parameter in Parameters)
            {
                yield return parameter;
            }
        }


        public override TypeContainer ValueType { get; }
        internal override ScriptType Evaluate(EvaluationScope scope, Diagnostics diagnostics, ref EvaluationState state)
        {
            var variable = Identifier.Evaluate(scope, diagnostics, ref state);
            if (!(variable is ScriptFunction function))
            {
                throw new EvaluationException(state.Script.FileName, $"Variable is not a function.", Identifier.Span);
            }
            var ls = new List<ScriptType>();
            foreach (var t in Parameters)
            {
                var evaluate = t.Evaluate(scope, diagnostics, ref state);
                ls.Add(evaluate);
            }

            var val = function.Run(diagnostics, ls.ToArray(), scope, Span, state);
            return val;
        }
    }
}