using System;
using System.Text;
using Upsilon.Evaluator;
using Upsilon.Text;

namespace Upsilon.Exceptions
{
    public class EvaluationException : Exception
    {
        private readonly Stacktrace _stacktrace;
        public string FileName { get; }
        public string ErrorMessage { get; }
        public TextSpan Span { get; }

        public EvaluationException(string fileName, string message, TextSpan span, Stacktrace stacktrace = null)
        {
            _stacktrace  = stacktrace;
            FileName     = fileName;
            ErrorMessage = message;
            Span         = span;
        }

        public EvaluationException(string fileName, Exception inner, TextSpan span, Stacktrace stacktrace = null)
        : base("An exception occured in called C# code. See the inner exception for more details.", inner)
        {
            _stacktrace  = stacktrace;
            FileName     = fileName;
            ErrorMessage = Message;
            Span         = span;
        }

        public override string ToString()
        {
            var err = new StringBuilder($"[{FileName}] ({Span.StartLine},{Span.StartPosition}) {ErrorMessage}");
            if (_stacktrace != null)
            {
                foreach (var statement in _stacktrace.GetTopStack(10))
                {
                    err.Append($"\n{statement.Name}");
                }
            }
            return err.ToString();
        }

        public override string Message => ToString();
    }
}