using System;
using Upsilon.BaseTypes;
using Upsilon.BaseTypes.Number;

namespace Upsilon.StandardLibraries
{
    internal class StringLibrary
    {
        [ScriptFunction("byte", "Get the byte value of the first character of a string", directScriptManipulation: true)]
        public ScriptNumber Byte(ScriptString str)
        {
            return new ScriptNumberLong(str.Value[0]);
        }

        [ScriptFunction("byte", "Get the byte value of the first character of a string", directScriptManipulation: true)]
        public ScriptNumber Byte(ScriptString str, ScriptNumberLong index)
        {
            if (index < 1 || index > str.Value.Length)
                return null;
            var c = str.Value[(int) (index - 1)];
            return new ScriptNumberLong(c);
        }

        [ScriptFunction("len", "Return the length of the string passed.", directScriptManipulation: true)]
        public ScriptNumber Length(ScriptString str)
        {
            return str.Length();
        }

        [ScriptFunction("lower", "Make uppercase characters lower case.", directScriptManipulation: true)]
        public ScriptString Lower(ScriptString str)
        {
            return new ScriptString(str.Value.ToLowerInvariant());
        }

        [ScriptFunction("reverse", "Reverses a string.", directScriptManipulation: true)]
        public ScriptString Reverse(ScriptString str)
        {
            var cArr = str.Value.ToCharArray();
            Array.Reverse(cArr);
            return new ScriptString(new string(cArr));
        }

        [ScriptFunction("sub", "Make all the lower case characters upper case.", directScriptManipulation: true)]
        public ScriptString Substring(ScriptString str, ScriptNumberLong startIndex, ScriptNumberLong endIndex = null)
        {
            var s = str.Value;
            var start = startIndex.Value;
            if (start < 0)
                start = s.Length + start;
            if (endIndex == null)
            {
                return new ScriptString(str.Value.Substring((int) start));
            }
            var end = endIndex.Value;
            if (end < 0)
                end = s.Length + end;
            var length = end - start;
            return new ScriptString(str.Value.Substring((int) start, (int) length));
        }

        [ScriptFunction("upper", "Make all the lower case characters upper case.", directScriptManipulation: true)]
        public ScriptString Upper(ScriptString str)
        {
            return new ScriptString(str.Value.ToUpperInvariant());
        }
    }
}